package usi.dbdp.portal.task.service;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.ServletContext;
import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.stereotype.Service;
import org.springframework.web.context.ServletContextAware;
import usi.dbdp.portal.task.dao.TimeTaskDao;
import usi.dbdp.portal.task.dto.SysTimerDto;
import usi.dbdp.portal.task.service.TimeTaskService;
import usi.dbdp.portal.task.service.TimerJob;

@Service
public class QuartzService implements ServletContextAware {

   @Resource
   private TimeTaskDao jobTaskDao;
   @Resource
   private TimeTaskService jobTaskService;
   private ServletContext servletContext;


   public void setServletContext(ServletContext servletContext) {
      this.servletContext = servletContext;
   }

   @PostConstruct
   public void init() {
      try {
         StdSchedulerFactory e = new StdSchedulerFactory();
         Scheduler scheduler = e.getScheduler();
         this.servletContext.setAttribute("scheduler", scheduler);
         InetAddress addr = null;
         String ip = null;

         try {
            addr = InetAddress.getLocalHost();
            ip = addr.getHostAddress();
         } catch (UnknownHostException var6) {
            var6.printStackTrace();
         }

         System.out.println("服务器ip:" + ip);
         this.jobTaskDao.updateJobTask(0, ip);
      } catch (SchedulerException var7) {
         var7.printStackTrace();
      }

   }

   public Map startJob(SysTimerDto jobTaskDto) {
      Scheduler scheduler = (Scheduler)this.servletContext.getAttribute("scheduler");
      Boolean flag = Boolean.valueOf(true);
      String failCause = "";
      HashMap map = new HashMap();
      String jobName = jobTaskDto.getTimerId() + "Job";
      String groupName = jobTaskDto.getTimerId() + "Group";
      String triggerName = jobTaskDto.getTimerId() + "Trigger";
      JobDetail job = JobBuilder.newJob(TimerJob.class).withIdentity(jobName, groupName).build();
      job.getJobDataMap().put("jobTask", jobTaskDto);
      CronTrigger trigger = null;

      try {
         trigger = (CronTrigger)TriggerBuilder.newTrigger().withIdentity(triggerName, groupName).withSchedule(CronScheduleBuilder.cronSchedule(jobTaskDto.getCronExpression())).build();
      } catch (Exception var13) {
         var13.printStackTrace();
         map.put("flag", Boolean.valueOf(false));
         map.put("failCause", "非法的cron表达式！");
         return map;
      }

      try {
         if(scheduler.checkExists(JobKey.jobKey(jobName, groupName))) {
            map.put("flag", Boolean.valueOf(false));
            map.put("failCause", "已经被启动！");
            return map;
         }

         scheduler.scheduleJob(job, trigger);
         if(!scheduler.isStarted()) {
            scheduler.start();
         }

         flag = Boolean.valueOf(true);
         failCause = "启动成功！";
         this.jobTaskDao.updateJobTaskByJobId(jobTaskDto, 1);
      } catch (SchedulerException var12) {
         var12.printStackTrace();
         flag = Boolean.valueOf(false);
         failCause = "未知异常，启动失败！";
      }

      map.put("flag", flag);
      map.put("failCause", failCause);
      return map;
   }

   public Map stopJob(SysTimerDto jobTaskDto) {
      Scheduler scheduler = (Scheduler)this.servletContext.getAttribute("scheduler");
      HashMap map = new HashMap();
      Boolean flag = null;
      String failCause = "";
      String jobName = jobTaskDto.getTimerId() + "Job";
      String groupName = jobTaskDto.getTimerId() + "Group";
      String triggerName = jobTaskDto.getTimerId() + "Trigger";
      TriggerKey triggerKey = TriggerKey.triggerKey(triggerName, groupName);
      JobKey jobKey = JobKey.jobKey(jobName, groupName);

      try {
         if(!scheduler.checkExists(jobKey)) {
            map.put("flag", Boolean.valueOf(false));
            map.put("failCause", "已经被停止！");
            return map;
         }

         scheduler.pauseTrigger(triggerKey);
         scheduler.unscheduleJob(triggerKey);
         scheduler.deleteJob(jobKey);
         flag = Boolean.valueOf(true);
         failCause = "停止成功！";
         this.jobTaskDao.updateJobTaskByJobId(jobTaskDto, 0);
      } catch (Exception var12) {
         var12.printStackTrace();
         flag = Boolean.valueOf(false);
         failCause = "未知异常，停止失败！";
      }

      map.put("flag", flag);
      map.put("failCause", failCause);
      return map;
   }

   public void modifyJobTime(SysTimerDto jobTaskDto) {
      try {
         Scheduler e = (Scheduler)this.servletContext.getAttribute("scheduler");
         String jobName = jobTaskDto.getTimerId() + "Job";
         String groupName = jobTaskDto.getTimerId() + "Group";
         String triggerName = jobTaskDto.getTimerId() + "Trigger";
         TriggerKey triggerKey = TriggerKey.triggerKey(triggerName, groupName);
         JobKey jobKey = JobKey.jobKey(jobName, groupName);
         CronTrigger triggerOld = (CronTrigger)e.getTrigger(triggerKey);
         if(triggerOld != null) {
            String oldTime = triggerOld.getCronExpression();
            if(!oldTime.equalsIgnoreCase(jobTaskDto.getCronExpression())) {
               JobDetail job = e.getJobDetail(jobKey);
               e.pauseTrigger(triggerKey);
               e.unscheduleJob(triggerKey);
               e.deleteJob(jobKey);
               CronTrigger trigger1 = (CronTrigger)TriggerBuilder.newTrigger().withIdentity(triggerName, groupName).withSchedule(CronScheduleBuilder.cronSchedule(jobTaskDto.getCronExpression())).build();
               e.scheduleJob(job, trigger1);
            }

         }
      } catch (Exception var12) {
         throw new RuntimeException(var12);
      }
   }

   public static void main(String[] args) {
      InetAddress addr = null;

      try {
         addr = InetAddress.getLocalHost();
      } catch (UnknownHostException var3) {
         var3.printStackTrace();
      }

      System.out.println("host ip:" + addr.getHostAddress());
      System.out.println("host name:" + addr.getHostName());
   }
}
